@@ -10,25 +10,29 @@
 extern crate alloc;
 extern crate scrypt;
 extern crate base64;
+extern crate libsecp256k1_core;
+extern crate hmac;
+extern crate serde;
 
 mod unid;
 mod logger;
 mod allocator;
 mod handler;
+mod ffi;
+pub mod bindings;
 
 use core::lazy::Lazy;
 use alloc::string::{String, ToString};
+use alloc::format;
 use cstr_core::{CStr, CString, c_char};
 use logger::Logger;
 use spin::Mutex;
 use unid::utils::data_t::DataT;
-use unid::utils::aes_crypt::AesCrypt;
+use unid::{utils::{random, codec}, did::payload::{KeyPairSecp256K1, PublicKeyPayload, Payload}};
 
 #[cfg_attr(not(test), global_allocator)]
 static mut ALLOCATOR: allocator::ExternalHeap = allocator::ExternalHeap::empty();
 
-static mut AES_CRYPT: AesCrypt = AesCrypt::empty();
-
 #[repr(C)]
 pub struct UNiDConfig {
     client_id: *const c_char,
@@ -76,47 +80,175 @@ pub unsafe extern "C" fn unid_regist_handler_on_debug_message(handler: extern "C
     MUTEX_HANDLERS.lock().set_debug_message_handler(handler)
 }
 
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_crypto_trng(handler: extern "C" fn(u32) -> *mut c_char) {
+    MUTEX_HANDLERS.lock().set_crypto_trng(handler)
+}
+
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_https_post_request(handler: extern "C" fn(*mut c_char, *mut c_char, *mut c_char) -> *mut c_char) {
+    MUTEX_HANDLERS.lock().set_https_post_request(handler)
+}
+
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_aes_encryptor(handler: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32)) {
+    MUTEX_HANDLERS.lock().set_aes_encryptor_handler(handler)
+}
+
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_aes_decryptor(handler: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32)) {
+    MUTEX_HANDLERS.lock().set_aes_decryptor_handler(handler)
+}
+
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_ecdsa_signer(handler: extern "C" fn(*mut c_char, *mut u8, *mut c_char, *mut c_char)) {
+    MUTEX_HANDLERS.lock().set_ecdsa_signer_handler(handler)
+}
+
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_regist_handler_on_ecdsa_verifier(handler: extern "C" fn(*mut c_char, *mut c_char, *mut c_char, *mut c_char, *mut DataT, *mut i32)) {
+    MUTEX_HANDLERS.lock().set_ecdsa_verifier_handler(handler)
+}
+
 /// unid :: init
 /// 
 /// # Safety
 #[no_mangle]
-pub unsafe extern "C" fn unid_init(config: UNiDConfig) -> UNiDContext {
-    let _logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
-
+pub unsafe extern "C" fn unid_init() {
     let alloc_handler = MUTEX_HANDLERS.lock().get_memory_alloc_handler();
     let dealloc_handler = MUTEX_HANDLERS.lock().get_memory_dealloc_handler();
 
-    assert!(! alloc_handler.is_none());
-    assert!(! dealloc_handler.is_none());
+    assert!(alloc_handler.is_some());
+    assert!(dealloc_handler.is_some());
 
     ALLOCATOR.init(alloc_handler.unwrap(), dealloc_handler.unwrap());
 
     // build context then return
-    UNiDContext {
-        client_id    : config.client_id,
-        client_secret: config.client_secret,
-    }
+    // UNiDContext {
+    //     client_id    : config.client_id,
+    //     client_secret: config.client_secret,
+    // }
 }
 
 /// aes :: init
 /// 
 /// # Safety
-#[no_mangle]
-pub unsafe extern "C" fn aes_init(encryptor: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32), decryptor: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32)) {
-    AES_CRYPT.init(encryptor, decryptor);
+// #[no_mangle]
+// pub unsafe extern "C" fn aes_init(encryptor: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32), decryptor: extern "C" fn(*mut DataT, *mut DataT, *mut DataT, *mut u8, u32)) {
+//     AES_CRYPT.init(encryptor, decryptor);
+// }
+
+use core::convert::From;
+
+#[repr(C)]
+pub struct KeyRing {
+    sign_key: *mut c_char,
+    update_key: *mut c_char,
+    recovery_key: *mut c_char,
 }
 
+impl From<(*mut c_char, *mut c_char, *mut c_char)> for KeyRing {
+    fn from(tup: (*mut c_char, *mut c_char, *mut c_char)) -> KeyRing {
+        KeyRing {
+            sign_key: tup.0,
+            update_key: tup.1,
+            recovery_key: tup.2,
+        }
+    }
+}
+
+impl From<KeyRing> for (*mut c_char, *mut c_char, *mut c_char) {
+    fn from(tup: KeyRing) -> (*mut c_char, *mut c_char, *mut c_char) {
+        (tup.sign_key, tup.update_key, tup.recovery_key)
+    }
+}
+
+/// ecdsa :: init
+/// 
+/// # Safety
+// #[no_mangle]
+// pub unsafe extern "C" fn ecdsa_init(signer: extern "C" fn(*mut c_char, *mut u8, *mut c_char, *mut c_char), verifier: extern "C" fn(*mut c_char, *mut c_char, *mut c_char, *mut c_char, *mut DataT, *mut i32)) {
+//     ECDSA.init(signer, verifier);
+// }
+
+
 /// unid :: core :: create_did
 /// 
 /// # Safety
 #[no_mangle]
-pub unsafe extern "C" fn unid_core_create_did(_context: UNiDContext) -> *mut c_char {
-    let _logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+pub unsafe extern "C" fn unid_core_create_did() -> KeyRing {
+    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
 
-    let r = String::from("WIP_FOR_ROT");
-    let r_c_str = CString::new(r).unwrap();
+    let private_key = random::Random::trng_bytes(&32).unwrap();
 
-    r_c_str.into_raw()
+    let x = &private_key[0..16];
+    let y = &private_key[16..];
+
+    let k = KeyPairSecp256K1 {
+        kty: "EC".to_string(),
+        crv: "secp256k1".to_string(),
+        kid: None,
+        d: None,
+        x: codec::Base64Url::encode(&x.to_vec()),
+        y: codec::Base64Url::encode(&y.to_vec()),
+    };
+
+    let public_key = PublicKeyPayload {
+        r#id: "signing".to_string(),
+        r#type: "EcdsaSecp256k1VerificationKey2019".to_string(),
+        r#jwk: k.clone(),
+        purpose: ["".to_string()].to_vec()
+    };
+
+    let payload = match Payload::new(
+        &[public_key].to_vec(),
+        &k,
+        &k,
+    ) {
+        Ok(v) => v,
+        Err(_) => panic!()
+    };
+
+    logger.debug(format!("k = {:?}, payload = {:?}", k, serde_json::to_string(&payload).unwrap()));
+
+    let handler = MUTEX_HANDLERS.lock().get_https_post_request();
+
+    if let Some(..) = handler {
+        let host = match ffi::Ffi::string_to_ptr("did.getunid.io".to_string()) {
+            Ok(v) => v,
+            Err(_) => panic!()
+        };
+        let path = match ffi::Ffi::string_to_ptr("/api/v1/operations".to_string()) {
+            Ok(v) => v,
+            Err(_) => panic!()
+        };
+        let body = match ffi::Ffi::string_to_ptr(serde_json::to_string(&payload).unwrap()) {
+            Ok(v) => v,
+            Err(_) => panic!()
+        };
+
+        let response = handler.unwrap()(host, path, body);
+
+        logger.info("-----".to_string());
+        logger.info(format!("create did = {:?}", ffi::Ffi::string_from_ptr(response).unwrap()));
+        logger.info("-----".to_string());
+
+        ffi::Ffi::disposer(host);
+        ffi::Ffi::disposer(path);
+        ffi::Ffi::disposer(body);
+    }
+
+    (
+        ffi::Ffi::binary_to_ptr(&private_key).unwrap(),
+        ffi::Ffi::binary_to_ptr(&private_key).unwrap(),
+        ffi::Ffi::binary_to_ptr(&private_key).unwrap(),
+    ).into()
 }
 
 /// unid :: core :: resolve_did
@@ -250,7 +382,7 @@ pub unsafe extern "C" fn unid_utils_codec_base64_encode(content: *const c_char)
     };
     let v1_str = v1.to_str().unwrap().to_string();
 
-    let r = unid::utils::codec::Codec::base64_encode(v1_str);
+    let r = unid::utils::codec::Base64Url::encode(&v1_str.as_bytes().to_vec());
     let r_c_str = CString::new(r).unwrap();
 
     r_c_str.into_raw()
@@ -270,8 +402,8 @@ pub unsafe extern "C" fn unid_utils_codec_base64_decode(content: *const c_char)
     };
     let v1_str = v1.to_str().unwrap().to_string();
 
-    let r = unid::utils::codec::Codec::base64_decode(v1_str);
-    let r_c_str = CString::new(r).unwrap();
+    let r = unid::utils::codec::Base64Url::decode_as_string(&v1_str);
+    let r_c_str = CString::new(r.unwrap()).unwrap();
 
     r_c_str.into_raw()
 }
@@ -281,7 +413,9 @@ pub unsafe extern "C" fn unid_utils_codec_base64_decode(content: *const c_char)
 /// # Safety
 #[no_mangle]
 pub unsafe extern "C" fn unid_utils_multihasher_hash(_content: *const c_char) -> *mut c_char {
-    let _logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+
+    logger.info("Hello from Rust !");
 
     let r = String::from("WIP_FOR_ROT");
     let r_c_str = CString::new(r).unwrap();
@@ -289,30 +423,104 @@ pub unsafe extern "C" fn unid_utils_multihasher_hash(_content: *const c_char) ->
     r_c_str.into_raw()
 }
 
+
+/// unid :: ciphers :: signer :: kp_gen
+/// 
+/// # Safety
+#[no_mangle]
+pub unsafe extern "C" fn unid_ciphers_signer_kp_gen() -> *mut c_char {
+    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+
+    logger.debug("(BEGIN) unid_ciphers_signer_kp_gen");
+
+    // result
+    // let r = unid::ciphers::signer::Signer::kp_gen();
+    let r = "".to_string();
+    let r_c_str = CString::new(r).unwrap();
+    let r_ptr = r_c_str.into_raw();
+
+    logger.debug("( END ) unid_ciphers_signer_kp_gen");
+
+    r_ptr
+}
+
 /// unid :: ciphers :: signer :: sign
 /// 
 /// # Safety
 #[no_mangle]
-pub unsafe extern "C" fn unid_ciphers_signer_sign() -> *mut c_char {
-    let _logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+pub unsafe extern "C" fn unid_ciphers_signer_sign(message: *const c_char, secret_key64: *const c_char) -> *mut c_char {
+    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
 
-    let r = String::from("WIP_FOR_ROT");
+    logger.debug("(BEGIN) unid_ciphers_signer_sign");
+
+
+    // v1
+    let v1 = {
+        assert!(! message.is_null());
+
+        CStr::from_ptr(message)
+    };
+    let v1_str = v1.to_str().unwrap().to_string();
+    logger.debug(alloc::format!("message to sign = {}", v1_str));
+
+    // v2
+    let v2 = {
+        assert!(! secret_key64.is_null());
+
+        CStr::from_ptr(secret_key64)
+    };
+    let v2_str = v2.to_str().unwrap().to_string();
+
+    // result
+    let r = unid::ciphers::signer::Signer::signew(v1_str, v2_str);
     let r_c_str = CString::new(r).unwrap();
+    let r_ptr = r_c_str.into_raw();
 
-    r_c_str.into_raw()
+    logger.debug("( END ) unid_ciphers_signer_sign");
+
+    r_ptr
 }
 
 /// unid :: ciphers :: signer :: verify
 /// 
 /// # Safety
 #[no_mangle]
-pub unsafe extern "C" fn unid_ciphers_signer_verify() -> *mut c_char {
-    let _logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
+pub unsafe extern "C" fn unid_ciphers_signer_verify(message: *const c_char, signature64: *const c_char, pub_key64: *const c_char) -> bool {
+    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
 
-    let r = String::from("WIP_FOR_ROT");
-    let r_c_str = CString::new(r).unwrap();
+    logger.debug("(BEGIN) unid_ciphers_signer_verify");
 
-    r_c_str.into_raw()
+    // v1
+    let v1 = {
+        assert!(! message.is_null());
+
+        CStr::from_ptr(message)
+    };
+    let v1_str = v1.to_str().unwrap().to_string();
+    logger.debug(alloc::format!("message to verify = {}", v1_str));
+
+    // v2
+    let v2 = {
+        assert!(! signature64.is_null());
+
+        CStr::from_ptr(signature64)
+    };
+    let v2_str = v2.to_str().unwrap().to_string();
+
+    // v3
+    let v3 = {
+        assert!(! pub_key64.is_null());
+
+        CStr::from_ptr(pub_key64)
+    };
+    let v3_str = v3.to_str().unwrap().to_string();
+
+    // result
+    let r_value = unid::ciphers::signer::Signer::verifynew(v1_str, v2_str, v3_str);
+
+    logger.debug("( END ) unid_ciphers_signer_verify");
+
+    r_value
 }
 
 /// unid :: ciphers :: cipher :: encrypt
@@ -344,7 +552,7 @@ pub unsafe extern "C" fn unid_ciphers_cipher_encrypt(plaintext: *const c_char, s
 
     // result
 
-    let r = unid::ciphers::cipher::Cipher::encrypt(v1_str, v2_str);
+    let r = String::from(""); // unid::ciphers::cipher::Cipher::encrypt(v1_str, v2_str);
 
     let r_c_str = CString::new(r).unwrap();
     let r_ptr = r_c_str.into_raw();
@@ -383,7 +591,7 @@ pub unsafe extern "C" fn unid_ciphers_cipher_decrypt(buffered_ciphertext_base64:
 
     // result
 
-    let r = unid::ciphers::cipher::Cipher::decrypt(v1_str, v2_str);
+    let r = String::from(""); // unid::ciphers::cipher::Cipher::decrypt(v1_str, v2_str);
 
     let r_c_str = CString::new(r).unwrap();
     let r_ptr = r_c_str.into_raw();
@@ -393,82 +601,6 @@ pub unsafe extern "C" fn unid_ciphers_cipher_decrypt(buffered_ciphertext_base64:
     r_ptr
 }
 
-/// unid :: ciphers :: hasher :: digest
-/// 
-/// # Safety
-#[no_mangle]
-pub unsafe extern "C" fn unid_ciphers_hasher_digest(content: *const c_char, secret: *const c_char) -> *mut c_char {
-    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
-
-    logger.debug("(BEGIN) unid_ciphers_hasher_digest");
-
-    // v1
-    let v1 = {
-        assert!(! content.is_null());
-
-        CStr::from_ptr(content)
-    };
-    let v1_str = v1.to_str().unwrap().to_string();
-
-    // v2
-    let v2 = {
-        assert!(! secret.is_null());
-
-        CStr::from_ptr(secret)
-    };
-    let v2_str = v2.to_str().unwrap().to_string();
-
-    // result
-    let r = unid::ciphers::hasher::Hasher::digest(v1_str, v2_str);
-    let r_c_str = CString::new(r).unwrap();
-    let r_ptr = r_c_str.into_raw();
-
-    logger.debug("( END ) unid_ciphers_hasher_digest");
-
-    r_ptr
-}
-
-/// unid :: ciphers :: hasher :: verify
-/// 
-/// # Safety
-#[no_mangle]
-pub unsafe extern "C" fn unid_ciphers_hasher_verify(content: *const c_char, digest: *const c_char, secret: *const c_char) -> bool {
-    let logger = Logger::new(MUTEX_HANDLERS.lock().get_debug_message_handler());
-
-    logger.debug("(BEGIN) unid_ciphers_hasher_verify");
-
-    // v1
-    let v1 = {
-        assert!(! content.is_null());
-
-        CStr::from_ptr(content)
-    };
-    let v1_str = v1.to_str().unwrap().to_string();
-
-    // v2
-    let v2 = {
-        assert!(! digest.is_null());
-
-        CStr::from_ptr(digest)
-    };
-    let v2_str = v2.to_str().unwrap().to_string();
-
-    // v3
-    let v3 = {
-        assert!(! secret.is_null());
-
-        CStr::from_ptr(secret)
-    };
-    let v3_str = v3.to_str().unwrap().to_string();
-
-    // result
-    let r_value = unid::ciphers::hasher::Hasher::verify(v1_str, v2_str, v3_str);
-
-    logger.debug("( END ) unid_ciphers_hasher_verify");
-
-    r_value
-}
-
 #[cfg(not(test))]
 use core::panic::PanicInfo;
 
@@ -480,48 +612,8 @@ pub extern "C" fn panic(_panic: &PanicInfo<'_>) -> ! {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-
     extern crate std;
 
     #[cfg_attr(test, global_allocator)]
     static mut A: std::alloc::System = std::alloc::System;
-
-    #[test]
-    fn test_unid_ciphers_hasher_digest() {
-        let content = CString::new("content");
-        let secret = CString::new("secret");
-
-        unsafe {
-            let c_ptr = unid_ciphers_hasher_digest(content.unwrap().as_ptr(), secret.unwrap().as_ptr());
-            let c_str = CStr::from_ptr(c_ptr);
-
-            assert_eq!(
-                c_str.to_str().unwrap(),
-                "pfMFlg7ax3Oka6O6FiWJxyAEVels4EOHUWVIgL8YXW21G+BkA5KTxCSJGnpd7hfAsodxp0Cu2Oa2uXdwqmOmXQ=="
-            );
-
-            // dispose!
-            unid_disposer(c_ptr);
-        }
-    }
-
-    #[test]
-    fn test_unid_ciphers_hasher_verify() {
-        let content = CString::new("content");
-        let secret = CString::new("secret");
-        let digest = CString::new("pfMFlg7ax3Oka6O6FiWJxyAEVels4EOHUWVIgL8YXW21G+BkA5KTxCSJGnpd7hfAsodxp0Cu2Oa2uXdwqmOmXQ==");
-
-        unsafe {
-            let is_verified = unid_ciphers_hasher_verify(content.unwrap().as_ptr(), digest.unwrap().as_ptr(), secret.unwrap().as_ptr());
-
-            assert!(is_verified);
-        }
-    }
-
-    #[allow(clippy::eq_op)]
-    #[test]
-    fn it_works() {
-        assert_eq!("hello", "hello");
-    }
 }
