@@ -1,5 +1,6 @@
 pub mod ciphers;
 // pub mod core;
-// pub mod did_unid;
+pub mod did;
 pub mod runtime;
-pub mod utils;
\ No newline at end of file
+pub mod utils;
+pub mod errors;
\ No newline at end of file
