@@ -1,3 +1,3 @@
 // pub mod bip39;
 // pub mod bip32;
-// pub mod secp256k1;
\ No newline at end of file
+pub mod secp256k1;
\ No newline at end of file
