@@ -1,6 +1,10 @@
 pub mod codec;
-pub mod data_t;
-pub mod aes_crypt;
 pub mod random;
 // pub mod http_client;
-// pub mod multihasher;
\ No newline at end of file
+// pub mod multihasher;
+pub mod secp256k1;
+pub mod ecdsa;
+pub mod multihasher;
+pub mod hmac;
+pub mod data_t;
+pub mod aes_crypt;
